from os import getenv


class EnvironmentVariableNotSet(Exception):
    message = ""

    def __init__(self, environment_variable: str, *args, **kwargs):
        self.message = "Environment variable " + environment_variable + " is not set."

        if isinstance(args, list) and len(args) > 0:
            self.message = str(args.pop(0))

        for key in list(kwargs.keys()):
            setattr(self, key, kwargs.pop(key))

        if not self.message:
            self.message = (
                f"{getattr(self, 'reason', 'Unknown')} "
                + f"({getattr(self, 'error_type', vars(self))})"
            )

        super().__init__(self.message, *args, **kwargs)

    def __str__(self):
        return self.message


def get_environment_variable(key: str) -> str:
    """
    Getting environment variable by key. Raises exception when variables is not set.
    """

    environment_variable = getenv(key)

    if environment_variable is None or environment_variable == "":
        raise EnvironmentVariableNotSet(key)

    return environment_variable


__all__ = ["get_environment_variable"]



from os import getenv


def boolean_environment_value(name: str, default_value: bool | None = None) -> bool:
    """
    Parses an environment variable to boolean
    """

    true_ = (
        "true",
        "1",
        "t",
        "yes",
        "on",
    )  # Add more entries if you want, like: `y`, `yes`, `on`, ...

    false_ = (
        "false",
        "0",
        "f",
        "no",
        "off",
    )  # Add more entries if you want, like: `n`, `no`, `off`, ...

    value = getenv(name)

    if value is None:
        if default_value is None:
            raise ValueError(f"Environment variable `{name}` not set!")

        return default_value

    if value.lower() not in true_ + false_:
        return False

    return value.lower() in true_



import json

from pydantic import BaseModel

from nwon_baseline.typings import AnyDict


def pydantic_model_to_dict(model: BaseModel) -> AnyDict:
    return json.loads(model.model_dump_json())


__all__ = ["pydantic_model_to_dict"]



from nwon_baseline.shell_helper.boolean_environment_value import (
    boolean_environment_value,
)
from nwon_baseline.shell_helper.execute_command import (
    execute_command,
    execute_command_and_print,
)
from nwon_baseline.shell_helper.get_environment_variable import get_environment_variable

__all__ = [
    "execute_command",
    "execute_command_and_print",
    "get_environment_variable",
    "boolean_environment_value",
]



import json
from typing import Type

from pydantic import BaseModel

from nwon_baseline.pydantic.path_to_write_pydantic_model import (
    path_to_write_pydantic_model,
)


def save_pydantic_model_schema(pydantic_model: Type[BaseModel], file_path: str) -> str:
    """
    Saves the JSON schema for a Pydantic Model in a json file.

    Expects the file_path to end with '.json'.
    If the suffix is not json we take the basename of the file_path and attach .json
    """

    file_path = path_to_write_pydantic_model(file_path)

    with open(file_path, "w+", encoding="utf-8") as outfile:
        outfile.write(json.dumps(pydantic_model.model_json_schema()))

    return file_path
